@@ -1,6 +1,6 @@
 import httplib
 
-_EXCEPTION_REPR_TEMPLATE = """HTTP Error {e.code} ({e.code_string}):
+_EXCEPTION_REPR_TEMPLATE = """HTTP {e.code} ({e.code_string}):
   METHOD: {e.method}
   URL: {e.url}
   SENT DATA: {e.sent_data}
@@ -8,7 +8,7 @@ _EXCEPTION_REPR_TEMPLATE = """HTTP Error {e.code} ({e.code_string}):
 """
 
 class JSONRestRequestException(Exception):
-    def __init__(self, method, url, code, sent_headers, sent_data, received_headers, received_data):
+    def __init__(self, method, url, code, sent_headers, sent_data, received_headers, received_data, msg=None):
         super(JSONRestRequestException, self).__init__()
         self.method = method
         self.url = url
@@ -18,17 +18,22 @@ class JSONRestRequestException(Exception):
         self.sent_headers = sent_headers
         self.received_data = received_data
         self.received_headers = received_headers
+        self.msg = msg
     @classmethod
-    def from_request_and_http_error(cls, request, http_error, sent_data, error_data):
+    def from_request_and_response(cls, request, response, sent_data, received_data, msg=None):
         return cls(
             method=request.get_method(),
             url=request.get_full_url(),
-            code=http_error.code,
+            code=response.code,
             sent_headers=request.headers,
             sent_data=sent_data,
-            received_headers=http_error.headers,
-            received_data=error_data)
+            received_headers=response.headers,
+            received_data=received_data,
+            msg=msg)
     def __repr__(self):
         return _EXCEPTION_REPR_TEMPLATE.format(e=self)
     def __str__(self):
         return repr(self)
+
+class JSONRestDecodeException(JSONRestRequestException):
+    pass
